// $Id: padma.js,v 1.3.2.8 2005/07/11 17:42:48 vnagarjuna Exp $ -->

//Copyright 2005 Nagarjuna Venna <vnagarjuna@yahoo.com>

/* ***** BEGIN LICENSE BLOCK ***** 
 *  This file is part of Padma.
 *
 *  Padma is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Padma is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Padma; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * ***** END LICENSE BLOCK ***** */

//Browser related

function getTransformer(inputMethod, outputMethod, rtsStartMode)
{
    var transformer = Transformer.createTransformer(inputMethod, outputMethod);
    if (inputMethod == Transformer.method_RTS)
        transformer.setRTSMode(rtsStartMode);
    if (outputMethod == Transformer.method_RTS) {
        var prefObj = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
        var branch = prefObj.getBranch("Padma.");
        var RTSOutputFormat = branch.prefHasUserValue("RTSOutputFormat")  ? branch.getIntPref("RTSOutputFormat") : 0; //default = 0
        transformer.setRTSOutputFormat(RTSOutputFormat > 2 || RTSOutputFormat < 0 ? 0 : RTSOutputFormat);
    }

    return transformer;
}

//Based on input method, determine if the selected text needs to be transformed
function requiresTransform(transformer, node)
{
    var mode = transformer.getInputMethod();
    if (mode == Transformer.method_Unknown) {
        if (node.nodeValue.match(/[\u0C00-\u0C7F]/))
            return true;
    }

    if (mode == Transformer.method_DynFonts || mode == Transformer.method_Unknown) {
        //Check computed style for parent node
        var parent = node.parentNode;
        while (parent.nodeType != 1 && parent.nodeType != 9)
            parent = parent.parentNode;
        
        if (parent.nodeType == 1) {
            var style = document.defaultView.getComputedStyle(parent, null);
            var font = style.getPropertyValue("font-family");
            return transformer.setDynamicFont(font);
        }
        return false;
    }
    return true;
}

//Process all nodes with root "root"
function processNodes(transformer, selection, root, start, end, soff, eoff)
{
    if (root.nodeType == 3) {
        if (selection.containsNode(root, true) && requiresTransform(transformer, root)) {
            if (root == start && root == end)
                root.replaceData(soff, eoff - soff, transformer.convert(root.nodeValue.substring(soff, eoff)));
            else if (root == start)
                root.replaceData(soff, root.length - soff, transformer.convert(root.nodeValue.substring(soff)));
            else if (root == end)
                root.replaceData(0, eoff, transformer.convert(root.nodeValue.substring(0, eoff)));
            else root.replaceData(0, root.length, transformer.convert(root.nodeValue));
        }
    }
    else {
        for(var i = 0; i < root.childNodes.length; ++i) {
            processNodes(transformer, selection, root.childNodes.item(i), start, end, soff, eoff);
        }
    }
}

function traverseDOM(transformer, selection)
{
    for(var i = 0; i < selection.rangeCount; i++) {
        var range = selection.getRangeAt(i);
        processNodes(transformer, selection, range.commonAncestorContainer, range.startContainer, 
                    range.endContainer, range.startOffset, range.endOffset);
    }
}

//Used only for special controls
function setTransformedText(text)
{
    //The credit for this piece goes to Jed Brown of the BBCode extension.
    //http://jedbrown.net/1.0
    try {
        var command = "cmd_insertText";
        var controller = document.commandDispatcher.getControllerForCommand(command);
        
        if (controller && controller.isCommandEnabled(command)) {
            controller = controller.QueryInterface(Components.interfaces.nsICommandController);
            var params = Components.classes["@mozilla.org/embedcomp/command-params;1"];
            params = params.createInstance(Components.interfaces.nsICommandParams);
            
            params.setStringValue("state_data", text);
            controller.doCommandWithParams(command, params);
        }
    } catch (e) {
        dump("Can't do cmd_insertText! ");
        dump(e+"\n");
    }
}

//Callback
function onTransform(inputMethod, outputMethod, rtsStartMode)
{
    initializeRelationships();

    //Create a transformer
    var transformer = getTransformer(inputMethod, outputMethod, rtsStartMode);
    
    var node = document.popupNode;
    var nodeLocalName = node.localName.toUpperCase();

    var selection;

    //Check for Selection in various controls
    //This code is thanks to Jaap Haitsma of the dictionarySearch extension and roachfiend of a million extensions
    //http://roachfiend.com/
    //http://mozdev.org/pipermail/project_owners/2004-March/001833.html
    if ((nodeLocalName == "TEXTAREA") || (nodeLocalName == "INPUT" && (node.type == "text" || node.type == "password"))) {
        selection = node.value.substring(node.selectionStart, node.selectionEnd);
    } else if (nodeLocalName == "OPTION") {
        var parentSelect = node.parentNode;
        if (parentSelect.localName.toUpperCase() == "SELECT") {
            if (parentSelect.multiple) {
                var anOption;
                for (var i = 0; i < parentSelect.options.length; i++) {
                    anOption = parentSelect.options[i];
                    if (anOption.selected) {
                        selection += " " + anOption.value;
                    }
                }
            } else {
                selection = node.value;
            }
        }
    } else if (nodeLocalName == "SELECT") {
        selection = node.options[node.selected].value;
    }
    else {
        //Selection in a regular window
        var focusedWindow = document.commandDispatcher.focusedWindow;
        var selection = focusedWindow.getSelection();
        traverseDOM(transformer, selection);
        selection.collapseToEnd();
        return;
    }

    //For the specials
    setTransformedText(transformer.convert(selection));
}
