// $Id: parser.js,v 1.1.2.3 2005/07/10 20:46:45 vnagarjuna Exp $ -->

//Copyright 2005 Nagarjuna Venna <vnagarjuna@yahoo.com>

/* ****** BEGIN LICENSE BLOCK ***** 
 *  This file is part of Padma.
 *
 *  Padma is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Padma is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Padma; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * ***** END LICENSE BLOCK ***** */

//A very simple minded parser that returns one character at a time
//This is the prototype for more specialized parsers
//Also includes the state machine

//Parser states
Parser.state_START  = 0;    //Syllable hunt starts from this state
Parser.state_CONSNT = 1;    //Found a consonant to start the syllable
Parser.state_PREFIX = 2;    //Found a prefix symbol

function Parser(input) {
    this.index = 0;
    this.length = input.length;
    this.text = input;
    this.state = Parser.state_START;
}

Parser.prototype.more = function ()
{
    return this.index < this.length;
}

Parser.prototype.next = function ()
{
    return this.text[this.index++];
}

//Change symbol type based on state
//No need to change symbol type by default
Parser.prototype.updateSymbolType = function (type)
{
    return type;
}

Parser.prototype.handleConsonantTermination = function (current)
{
    return current;
}

Parser.prototype.isCurrentSyllableComplete = function (type)
{
    return type == Padma.type_spaccu || type == Padma.type_gunimtam;
}

//State Machine
//The parser has just determined that symbol is a match for key
//The syllable is built into current and is returned from this function
Parser.prototype.handleInput = function (symbol, key, current)
{
    var old_type = Padma.type_unknown;
    if (symbol != null)
        old_type = Padma.getType(symbol);
    else symbol = key;

    var prefix = false;
    if (arguments.length == 4) {
        prefix = arguments[3];
    }

    //alert("state = " + this.state);
    //alert("type = " + old_type);
        
    var type = this.updateSymbolType(old_type);
    if (type != old_type) {
        symbol = Padma.dependentForm(symbol);
    }
    
    switch (this.state) {
        case Parser.state_START:
        case Parser.state_PREFIX: {
            this.index += key.length;
            if (prefix == true) {
                this.state = Parser.state_PREFIX;
                return symbol;
            }
            if (type == Padma.type_accu || type == Padma.type_spaccu || type == Padma.type_digit || type == Padma.type_unknown) {
                //Current syllable is complete
                this.state = Parser.state_START;
                //alert("1: returning " + symbol);
            }
            else if (type == Padma.type_hallu) {
                //Wait for more
                this.state = Parser.state_CONSNT;
            }
            else {
                //alert("2: returning " + symbol);
                this.state = Parser.state_START;
            }
            return symbol;
        }
        
        case Parser.state_CONSNT: {
            if (type == Padma.type_accu || type == Padma.type_digit || type == Padma.type_unknown || type == Padma.type_hallu || prefix) {
                //next syllable has started, process it again next time around.
                //TODO: cache this symbol
                this.state = Parser.state_START;
                return this.handleConsonantTermination(current);
            }
            this.index += key.length;
            if (this.isCurrentSyllableComplete(type)) {
                //Syllable is complete
                this.state = Parser.state_START;
                //alert("5: returning " + current + symbol);
            }
            //vattu
            return current + symbol;
        }
    }
}
