// $Id: Transformer.js,v 1.1.2.6 2005/07/10 20:46:46 vnagarjuna Exp $ -->

//Copyright 2005 Nagarjuna Venna <vnagarjuna@yahoo.com>

/* ****** BEGIN LICENSE BLOCK ***** 
 *  This file is part of Padma.
 *
 *  Padma is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Padma is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Padma; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * ***** END LICENSE BLOCK ***** */

//Transformer

//Input/output methods
Transformer.method_RTS      = 0;
Transformer.method_DynFonts = 1;
Transformer.method_Unicode  = 2;
Transformer.method_Unknown  = 3;

//Dynamic fonts supported
Transformer.dynFont_Unknown      = 0;
Transformer.dynFont_Eenadu       = 1;
Transformer.dynFont_ShreeTel0900 = 2;
Transformer.dynFont_Hemalatha    = 3;

Transformer.dynFont_name_Eenadu       = "EENADU";
Transformer.dynFont_name_ShreeTel0900 = "SHREE-TEL-0900";
Transformer.dynFont_name_Hemalatha    = "TLW-TTHEMALATHA";

Transformer.padmaDynFontList = new Array();
Transformer.padmaDynFontList[Transformer.dynFont_name_Eenadu]       = Transformer.dynFont_Eenadu ;
Transformer.padmaDynFontList[Transformer.dynFont_name_ShreeTel0900] = Transformer.dynFont_ShreeTel0900;
Transformer.padmaDynFontList[Transformer.dynFont_name_Hemalatha]    = Transformer.dynFont_Hemalatha;

function Transformer(input, output)
{
    this.input = input;
    this.output = output;
    this.dynFontIndex = Transformer.dynFont_Unknown;
    this.RTSOutputFormat = RTS;
}

Transformer.createTransformer = function (input, output)
{
    if (!Transformer.isValidOutputMethod(output) || !Transformer.isValidInputMethod(input))
        return null;
    if (input == Transformer.method_RTS)
        return new RTSTransformer(input, output);
    return new Transformer(input, output);
}

Transformer.prototype.getInputMethod = function ()
{
    return this.input;
}

Transformer.prototype.getOutputMethod = function ()
{
    return this.output;
}

Transformer.isValidInputMethod = function (method)
{
    return method == Transformer.method_RTS || method == Transformer.method_DynFonts || method == Transformer.method_Unicode 
           || method == Transformer.method_Unknown;
}

Transformer.isValidOutputMethod = function (method)
{
    return method == Transformer.method_RTS || method == Transformer.method_Unicode;
}

Transformer.prototype.convert = function (text)
{
    //Convert to internal format from inputMethod
    var output = "";
    var parser;
    if (this.input == Transformer.method_DynFonts || (this.input == Transformer.method_Unknown && this.dynFontIndex != Transformer.dynFont_Unknown)) {
        if (this.dynFontIndex == Transformer.dynFont_Eenadu)
            parser = new EenaduParser(text);
        else if (this.dynFontIndex == Transformer.dynFont_ShreeTel0900)
            parser = new DynamicFontParser(text, Shree_Tel_0900);
        else if (this.dynFontIndex == Transformer.dynFont_Hemalatha)
            parser = new DynamicFontParser(text, Hemalatha);
        this.dynFontIndex = Transformer.dynFont_Unknown;
    }
    else if (this.input == Transformer.method_Unicode || this.input == Transformer.method_Unknown) {
        parser = new UnicodeParser(text);
    }

    while (parser.more())
        output += this.toOutputFormat(parser.next());

    return output;
}

Transformer.prototype.toOutputFormat = function (syllable)
{
    var output = "";
    if (this.output == Transformer.method_Unicode) {
        for(var i = 0; i < syllable.length; i++) {
            output += Unicode.transformFromPadma(syllable[i]);
        }
    }
    else if (this.output == Transformer.method_RTS) {
        var last_type = Padma.getType(syllable[syllable.length - 1]);
        if (last_type == Padma.type_hallu || last_type == Padma.type_vattu)
            syllable += Padma.vowel_A;
        for(var i = 0; i < syllable.length; i++)
            output += this.RTSOutputFormat.transformFromPadma(syllable[i]);
    }
    return output;
}

Transformer.isSupportedDynamicFont = function (face)
{
    return Transformer.padmaDynFontList[face.toUpperCase()] != null;
}

Transformer.prototype.setDynamicFont = function (face)
{
    if (this.input != Transformer.method_DynFonts && this.input != Transformer.method_Unknown)
        return false;
    var index = Transformer.padmaDynFontList[face.toUpperCase()];
    if (index == null)
        return false;
    this.dynFontIndex = index;
    return true;
}

Transformer.prototype.setRTSOutputFormat = function (rtsType)
{
    this.RTSOutputFormat = RTS.getOutputFormat(rtsType);
}
